/*
 * Copyright (c) 2005-2013, BearWare.dk
 * 
 * Contact Information:
 *
 * Bjoern D. Rasmussen
 * Bedelundvej 79
 * DK-9830 Taars
 * Denmark
 * Email: contact@bearware.dk
 * Phone: +45 20 20 54 59
 * Web: http://www.bearware.dk
 *
 * This source code is part of the TeamTalk 4 SDK owned by
 * BearWare.dk. All copyright statements may not be removed 
 * or altered from any source distribution. If you use this
 * software in a product, an acknowledgment in the product 
 * documentation is required.
 *
 */

#include "textmessagedlg.h"
#include "mainwindow.h"
#include <QDebug>

#define LOCAL_TYPING_DELAY  5000
#define REMOTE_TYPING_DELAY 10000

extern TTInstance* ttInst;

TextMessageDlg::TextMessageDlg(int userid, QWidget * parent/* = 0*/)
: QDialog(parent, Qt::Dialog | Qt::WindowMinMaxButtonsHint | Qt::WindowSystemMenuHint)
, m_userid(userid)
, m_textchanged(false)
, m_remote_typing_id(0)
{
    init();
}

TextMessageDlg::TextMessageDlg(int userid, const textmessages_t& msgs,
                               QWidget * parent/* = 0*/)
: QDialog(parent, Qt::Dialog | Qt::WindowMinMaxButtonsHint | Qt::WindowSystemMenuHint)
, m_userid(userid)
, m_textchanged(false)
, m_remote_typing_id(0)
{
    init();
    
    textmessages_t::const_iterator ii = msgs.begin();
    for(;ii!=msgs.end();ii++)
        newMsg(*ii);
}

void TextMessageDlg::init()
{
    ui.setupUi(this);
    setWindowIcon(QIcon(APPICON));

    connect(ui.cancelButton, SIGNAL(clicked()), SLOT(slotCancel()));
    connect(ui.sendButton, SIGNAL(clicked()), SLOT(slotSendMsg()));
    connect(ui.newmsgTextEdit, SIGNAL(textChanged()), SLOT(slotTextChanged()));
    connect(ui.newmsgTextEdit, SIGNAL(sendTextMessage(const QString&)),
            SLOT(slotSendMsg(const QString&)));
    slotUpdateUser(m_userid, 0);
    slotTextChanged();

    m_local_typing_id = startTimer(LOCAL_TYPING_DELAY);
}

TextMessageDlg::~TextMessageDlg()
{
    emit(closedTextMessage(m_userid));
}

void TextMessageDlg::slotUpdateUser(int userid, int /*channelid*/)
{
    if(userid == m_userid)
    {
        User user;
        if(TT_GetUser(ttInst, userid, &user))
            this->setWindowTitle(tr("Messages") + " - " + _Q(user.szNickname));
    }
}

void TextMessageDlg::slotNewMessage(int userid, int msgid)
{
    if(userid == m_userid)
    {
        TextMessage msg;
        if(TT_GetTextMessage(ttInst, msgid, TRUE, &msg))
            newMsg(msg);
    }
}

void TextMessageDlg::slotUserLogout(int userid)
{
    if(m_userid == userid)
        ui.newmsgTextEdit->setDisabled(true);
}

void TextMessageDlg::timerEvent(QTimerEvent *event)
{
    if(m_local_typing_id == event->timerId())
    {
        if(m_textchanged)
        {
            ServerProperties srvprop;
            if(TT_GetServerProperties(ttInst, &srvprop) &&
               VersionSameOrLater(_Q(srvprop.szServerProtocolVersion), "4.4"))
            {
                TextMessage msg;
                ZERO_STRUCT(msg);
                msg.nFromUserID = TT_GetMyUserID(ttInst);
                msg.nMsgType = MSGTYPE_CUSTOM;
                msg.nToUserID = m_userid;
                QString cmd = QString("%1\r\n%2").arg(CMD_TYPING_TEXT)
                    .arg(!ui.newmsgTextEdit->toPlainText().isEmpty());
                COPY_TTSTR(msg.szMessage, cmd);
                TT_DoTextMessage(ttInst, &msg);
            }
            m_textchanged = false;
        }
    }

    if(m_remote_typing_id == event->timerId())
    {
        ui.newmsgLabel->setText(tr("New message"));
        killTimer(m_remote_typing_id);
        m_remote_typing_id = 0;
    }
}

void TextMessageDlg::slotCancel()
{
    this->close();
}

void TextMessageDlg::slotSendMsg()
{
    slotSendMsg(ui.newmsgTextEdit->toPlainText());
}

void TextMessageDlg::slotSendMsg(const QString& txt_msg)
{
    if(txt_msg.isEmpty())
        return;

    TextMessage msg;
    ZERO_STRUCT(msg);
    msg.nFromUserID = TT_GetMyUserID(ttInst);
    msg.nChannelID = 0;
    msg.nMsgType = MSGTYPE_USER;
    msg.nToUserID = m_userid;
    COPY_TTSTR(msg.szMessage, txt_msg);
    if(TT_DoTextMessage(ttInst, &msg)>0)
    {
        ui.newmsgTextEdit->setPlainText("");
        newMsg(msg);
        m_textchanged = false;
    }
}

void TextMessageDlg::slotTextChanged()
{
    ui.sendButton->setEnabled(ui.newmsgTextEdit->toPlainText().size()>0);
    m_textchanged = true;
}

void TextMessageDlg::newMsg(const TextMessage& msg)
{
    switch(msg.nMsgType)
    {
    case MSGTYPE_USER :
        ui.historyTextEdit->addTextMessage(msg);
        ui.newmsgLabel->setText(tr("New message"));
        break;
    case MSGTYPE_CUSTOM :
    {
        QStringList cmd_msg = _Q(msg.szMessage).split("\r\n");
        if(cmd_msg.size() >= 2 && cmd_msg[0] == CMD_TYPING_TEXT)
        {
            if(cmd_msg[1] == "1")
            {
                ui.newmsgLabel->setText(tr("New message - remote user typing."));
                if(m_remote_typing_id)
                    killTimer(m_remote_typing_id);
                m_remote_typing_id = startTimer(REMOTE_TYPING_DELAY);
            }
            else
            {
                if(m_remote_typing_id)
                    killTimer(m_remote_typing_id);
                m_remote_typing_id = 0;
                ui.newmsgLabel->setText(tr("New message"));
            }
        }
        break;
    }
    }
}
