/*
 * Copyright (c) 2005-2013, BearWare.dk
 * 
 * Contact Information:
 *
 * Bjoern D. Rasmussen
 * Bedelundvej 79
 * DK-9830 Taars
 * Denmark
 * Email: contact@bearware.dk
 * Phone: +45 20 20 54 59
 * Web: http://www.bearware.dk
 *
 * This source code is part of the TeamTalk 4 SDK owned by
 * BearWare.dk. All copyright statements may not be removed 
 * or altered from any source distribution. If you use this
 * software in a product, an acknowledgment in the product 
 * documentation is required.
 *
 */

#include "userdesktopwidget.h"
#include "common.h"
#include <QPaintEngine>
#include <QMouseEvent>

extern TTInstance* ttInst;

UserDesktopWidget::UserDesktopWidget(QWidget* parent, int userid, int border_width)
    : UserImageWidget(parent, userid, false, border_width)
    , m_desktop_updated(true)
    , m_mousedown(false)
{
    startTimer(100);
}

QSize UserDesktopWidget::imageSize() const
{
#ifdef USE_TT_PAINT
    DesktopWindow deskwnd = {0};
    int bmp_size;
    if(!TT_GetUserDesktopWindow(ttInst, m_userid, NULL, &bmp_size, &deskwnd))
        return QSize();
    return QSize(deskwnd.nWidth, deskwnd.nHeight);
#else
    return m_image.size();
#endif
}

void UserDesktopWidget::paintEvent(QPaintEvent *p)
{
    UserImageWidget::paintEvent(p);

    if(!m_cursorpos.isNull())
    {
        QPainter painter(this);
        QPoint topleft(m_cursorpos.x() - 3, m_cursorpos.y() - 3);
        QRect r(topleft, QSize(5, 5));
        painter.drawRect(r);
        topleft.setX(topleft.x() + 1);
        topleft.setY(topleft.y() + 1);
        r = QRect(topleft, QSize(4, 4));
        painter.fillRect(r, QBrush(QColor(128,255,86)));
    }
}


void UserDesktopWidget::timerEvent(QTimerEvent* e)
{
    Q_UNUSED(e);

    if(!m_desktop_updated)
        return;

#ifndef USE_TT_PAINT
    DesktopWindow deskwnd;
    ZERO_STRUCT(deskwnd);
    if(!TT_GetUserDesktopWindow(ttInst, m_userid, NULL, NULL, &deskwnd))
    {
        emit(userDesktopWindowEnded(m_userid));
        return;
    }

    QImage::Format fmt;
    switch(deskwnd.bmpFormat)
    {
    case BMP_RGB8_PALETTE :
        fmt = QImage::Format_Indexed8;
        break;
    case BMP_RGB16_555 :
        //fmt = QImage::Format_RGB16;
        fmt = QImage::Format_RGB555;
        break;
    case BMP_RGB24 :
        fmt = QImage::Format_RGB888;
        break;
    case BMP_RGB32 :
    default :
        fmt = QImage::Format_RGB32;
        break;
    }

    if(m_image.format() != fmt || m_image.width() != deskwnd.nWidth ||
       m_image.height() != deskwnd.nHeight)
    {
        m_image = QImage(deskwnd.nWidth, deskwnd.nHeight, fmt);

        if(deskwnd.bmpFormat == BMP_RGB8_PALETTE)
        {
            int bytes_per_line = m_image.bytesPerLine();
            bytes_per_line = bytes_per_line;
            QVector<QRgb> colors(256);
            for(int i=0;i<256;i++)
            {
                unsigned char* rgb = TT_Palette_GetColorTable(BMP_RGB8_PALETTE, i);
                //Qt swaps red and blue??
                colors[i] = qRgb( rgb[2], rgb[1], rgb[0]);
            }
            m_image.setColorTable(colors);
        }
        m_offset = QPoint(0,0);
    }

    int img_size = m_image.byteCount();
    TT_GetUserDesktopWindow(ttInst, m_userid,
        reinterpret_cast<char*>(m_image.bits()), &img_size, &deskwnd);

    //TODO: Qt swaps red and blue. Very ineffective way of getting around this.
    if(fmt == QImage::Format_RGB888)
        m_image = m_image.rgbSwapped();
#endif

    m_desktop_updated = false;

    update();
}

#ifdef USE_TT_PAINT
void UserDesktopWidget::runTTPaint(QPainter& painter)
{
    HDC hdc = painter.paintEngine()->getDC();
    QPoint p0 = this->mapTo(nativeParentWidget(), QPoint(0,0));

    //if(!TT_PaintDesktopWindow(ttInst, m_userid, hdc, p0.x(), p0.y(), width(), height()))
    //    emit(userDesktopWindowEnded(m_userid));

    QSize size = imageSize();
    int w = qMin(size.width(), width());
    int h = qMin(size.height(), height());

    if(!TT_PaintDesktopWindowEx(ttInst, m_userid, hdc, p0.x(), p0.y(), width(), height(),
                                0, 0, w, h))
        emit(userDesktopWindowEnded(m_userid));

    painter.paintEngine()->releaseDC(hdc);
}
#endif


void UserDesktopWidget::slotDesktopUpdate(int userid, int sessionid)
{
    Q_ASSERT(m_userid > 0);

    Q_UNUSED(sessionid);
    if(userid != m_userid)
        return;

    //Don't update display whenever the desktop window is updated.
    //It will take too much CPU.
    //Better to start a timer which detects changes
    m_desktop_updated = true;
}

void UserDesktopWidget::slotDesktopCursorUpdate(int src_userid,
                                                int dest_userid)
{
    if(dest_userid != m_userid)
        return;

    m_desktop_updated = true;

    int x, y;
    if(TT_GetUserDesktopCursor(ttInst, src_userid, dest_userid, &x, &y))
    {
        if(x < 0 || y < 0)
            m_cursorpos = QPoint();
        else
            m_cursorpos = QPoint(x, y);
    }

    update();
}

void UserDesktopWidget::mousePressEvent(QMouseEvent* event)
{
    UserImageWidget::mousePressEvent(event);
    m_mousedown = true;
    m_old_pos = event->pos();
}

void UserDesktopWidget::mouseReleaseEvent(QMouseEvent* event)
{
    UserImageWidget::mouseReleaseEvent(event);
    m_mousedown = false;
}

void UserDesktopWidget::mouseMoveEvent(QMouseEvent* event)
{
    UserImageWidget::mouseMoveEvent(event);

    QPoint pos = event->pos();

    if(m_mousedown)
    {
        int widget_width = this->width();
        int widget_height = this->height();

        int x_max = imageSize().width() - widget_width - 2 * m_border_width;
        int y_max = imageSize().height() - widget_height  - 2 * m_border_width;

        int x_diff = pos.x() - m_old_pos.x();
        int y_diff = pos.y() - m_old_pos.y();

        int new_x = m_offset.x() + x_diff * -1; 
        int new_y = m_offset.y() + y_diff * -1; 

        new_x = qMax(new_x, 0);
        new_x = qMin(new_x, x_max);
        new_y = qMax(new_y, 0);
        new_y = qMin(new_y, y_max);

        m_offset.setX(new_x);
        m_offset.setY(new_y);

        update();
    }
    m_old_pos = pos;
}
